//+------------------------------------------------------------------+
//|                                          VM_Money_Management.mq4 |
//|                                Copyright  2007, Victor Maranho |
//|                                              vmaranhao@gmail.com |
//+------------------------------------------------------------------+

//+---------------------------------------------------------------------------+
//| DEFINE O LOTE PARA O PAR                                                  |
//+---------------------------------------------------------------------------+

double VM_MM_Get_Lot_Size (double cotacaoBase, int    stopLoss      , 
                           double maxRisk    , double cotacaoCross=0)
  {
   
//   if (IsTesting()) { FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "VM_MM_Get_Lot_Size");
//                      FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "VM_MM_Get_Lot_Size - Cotao Base : " + cotacaoBase);
//                      FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "VM_MM_Get_Lot_Size - Cotao Cross: " + cotacaoCross); }
   double lots;
   int    lotsI;
   int    lotDig   = VM_MM_Get_Lot_Digits(cIxPar);
   double lotStp   = MarketInfo(cParNome[cIxPar], MODE_LOTSTEP);
   double minLot   = MarketInfo(cParNome[cIxPar], MODE_MINLOT);
   double maxLot   = MarketInfo(cParNome[cIxPar], MODE_MAXLOT);
   double pipVal   = VM_MM_Get_Pip_USD(cotacaoBase, cotacaoCross);
   int    minLotI;
   int    lotStpI;

   lots = NormalizeDouble (maxRisk / (pipVal * stopLoss), lotDig);
   
   if (lots < minLot) lots = 0;
   if (lots > maxLot) lots = maxLot;
   
   lotsI   = lots   * MathPow(10,lotDig);
   minLotI = minLot * MathPow(10,lotDig);
   lotStpI = lotStp * MathPow(10,lotDig);
        
   while ((lotsI - minLotI) % lotStpI != 0)
     {
      lots = lots - lotStp;
      if (lots < 0) 
        {
         lots = 0;
         break;
        }
      lotsI = lots * MathPow(10,lotDig);
     } 

//   if (IsTesting()) 
//     { FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "VM_MM_Get_Lot_Size - Lotes: " + lots);
//       FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "VM_MM_Get_Lot_Size - Lote Dig: " + lotDig);
//       FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "VM_MM_Get_Lot_Size - Lote Max: " + maxLot);
//       FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "VM_MM_Get_Lot_Size - Lote Min: " + minLot);
//       FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "VM_MM_Get_Lot_Size - Lote Step: " + lotStp);
//       FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "VM_MM_Get_Lot_Size - Valor Pip: " + pipVal);
//       FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "--------------------------------------------------"); }
   
   return (lots);
   
  }
  
//+---------------------------------------------------------------------------+
//| DETERMINA VALR DO PIP EM USD                                              |
//+---------------------------------------------------------------------------+

double VM_MM_Get_Pip_USD (double cotacaoBase, double cotacaoCross = 0)

  {
    int    acao;
    int    numDigs;
    double lotSize;
    double pipBase;

//   if (IsTesting()) 
//     { FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "VM_MM_Get_Pip_USD");
//       FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "VM_MM_Get_Pip_USD - Cotao Base : " + cotacaoBase);
//       FileWrite(tHandle,TimeToStr(TimeCurrent()) + tN04 + "VM_MM_Get_Pip_USD - Cotao Cross: " + cotacaoCross); }

   acao = cParCalCr[cIxPar];

   numDigs = MarketInfo(cParNome[cIxPar], MODE_DIGITS );
   lotSize = MarketInfo(cParNome[cIxPar], MODE_LOTSIZE);
   
   pipBase = (MathPow(10,-numDigs) / cotacaoBase) * lotSize;

   switch (acao)
     {
      case cAcao_Base_Par1_Eh_USD :                                  //  Pares USDxxx

           return (NormalizeDouble(pipBase, numDigs));

      case cAcao_Base_Par2_Eh_USD :                                  // Pares xxxUSD

           return (1.0);
           
      case cAcao_Crss_Par1_Eh_USD :                                  // Pares xxxyyy com cross = USDxxx
          
           return (NormalizeDouble((1 / cotacaoCross) * pipBase, 4));
           
      case cAcao_Crss_Par2_Eh_USD :                                  // Pares xxxyyy com cross = xxxUSD
          
           return (NormalizeDouble(pipBase * cotacaoCross, numDigs));
           
     }    

  }

//+---------------------------------------------------------------------------+
//| DETERMINA O NMERO DE DECIMAIS DO LOTE MNIMO                             |
//+---------------------------------------------------------------------------+

int VM_MM_Get_Lot_Digits(int ixPar)

  {
   
   int    digM;
   int    digS;
   double minLot;
   double lotStp;
   int    restoI;
   double restoD;
   
   digM   = 0;
   minLot = MarketInfo (cParNome[ixPar], MODE_MINLOT);
   
   restoD = (minLot * MathPow(10,digM));
   restoI = restoD;
   
   while ( restoD - restoI != 0 ) 
     {
      digM++;
      restoD = (minLot * MathPow(10,digM));
      restoI = restoD;
     }
   
   digS = 0;
   
   lotStp = MarketInfo (cParNome[ixPar], MODE_LOTSTEP);
   
   restoD = (lotStp * MathPow(10,digS));
   restoI = restoD;
   
   while ( restoD - restoI != 0 ) 
     {
      digS++;
      restoD = (minLot * MathPow(10,digS));
      restoI = restoD;
     }
   
   if (digM > digS)
     return (digM);
   else
     return (digS);
   
  }
  
//+---------------------------------------------------------------------------+
//| FIM                                                                       |
//+---------------------------------------------------------------------------+
  

